local Type, Version = "UIF-Font", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

local LSM = LibStub("LibSharedMedia-3.0")

local pairs = pairs

local CreateFrame, UIParent = CreateFrame, UIParent

local FontFlags = {
	["NONE"] = "None",
	["OUTLINE"] = "Thin",
	["THICKOUTLINE"] = "Thick",
	["MONOCHROME"] = "Monochrome",
}

local FontFrameHeight = 35
local FontFramePadding = 2
local FontFrameBackgroundEven = {0.1, 0.1, 0.1, 1}
local FontFrameBackgroundOdd = {0.15, 0.15, 0.15, 1}
local FontFrameHover = {0.23, 0.23, 0.0, 1}
local FontFrameChecked = {0.13, 0.33, 0, 1}

local FontFrames = {}

--[[-----------------------------------------------------------------------------
Font Frames
-------------------------------------------------------------------------------]]
local function FontFrame_OnEnter(self)
	self.bg:SetTexture(unpack(FontFrameHover))
end

local function FontFrame_OnLeave(self)	
	if self.checked then
		self.bg:SetTexture(unpack(FontFrameChecked))
	else
		if self.even then
			self.bg:SetTexture(unpack(FontFrameBackgroundEven))
		else
			self.bg:SetTexture(unpack(FontFrameBackgroundOdd))
		end
	end
end

local function FontFrame_OnClick(self, button)
	for i, FontFrame in pairs(self.obj.FontFrames) do
		if FontFrame ~= self then	
			FontFrame.checked = false
			if FontFrame.even then
				FontFrame.bg:SetTexture(unpack(FontFrameBackgroundEven))
			else
				FontFrame.bg:SetTexture(unpack(FontFrameBackgroundOdd))
			end
		end
	end
	self.checked = true
	self.bg:SetTexture(unpack(FontFrameChecked))
	if self.obj.DBObject then
		self.obj.DBObject.Font = self.font
	end
	self.obj:Fire("OnFontChanged", self.font)
	self.obj:Fire("OnValuesChanged", value)
	self.obj:SetFont(self.font)
end

local TestFontObject = CreateFont("UIF-TestFont")
local function CreateFontFrame(self, font, even)
	local FontFace = LSM:Fetch("font", font)
	if TestFontObject:SetFont(FontFace, 12, "NONE") then
		local frame = table.remove(FontFrames)
		if not frame then 
			frame = CreateFrame("Button")
			frame:EnableMouse(true)
			frame:RegisterForClicks("LeftButtonDown")
			frame:SetScript("OnEnter", FontFrame_OnEnter)
			frame:SetScript("OnLeave", FontFrame_OnLeave)
			frame:SetScript("OnClick", FontFrame_OnClick)
			
			local t1 = frame:CreateFontString(nil, "OVERLAY")
			t1:SetPoint("TOPLEFT", 0, -2)
			t1:SetPoint("RIGHT")
			t1:SetPoint("BOTTOM", frame, "CENTER", 0, 0)
			
			local t2 = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
			t2:SetPoint("TOPLEFT", t1, "BOTTOMLEFT")
			t2:SetPoint("BOTTOMRIGHT", 0, 2)
			
			local framebg = frame:CreateTexture(nil, "BACKGROUND")
			framebg:SetAllPoints()			
			
			frame.t1 = t1
			frame.t2 = t2
			frame.bg = framebg
		end
		
		frame.even = even
		if even then
			frame.bg:SetTexture(unpack(FontFrameBackgroundEven))
		else
			frame.bg:SetTexture(unpack(FontFrameBackgroundOdd))
		end
		frame.font = font
		frame.obj = self
		frame.t1:SetFont(FontFace, 14, "OUTLINE")
		frame.t1:SetText("1234567890!ABCabc")
		frame.t2:SetText(font)
		
		return frame
	end
end

local sortedFonts = {}
local function FontFrame_Show(this)
	local self = this.obj
	self.pullout:ClearAllPoints()
	if self.pullout:IsShown() then		
		AceGUI:ClearFocus()
	else
		self.pullout:SetPoint("BOTTOMRIGHT", self.prev, "TOPRIGHT")
		self.pullout:SetWidth(400)
		self.pullout:SetHeight(200)
		
		wipe(sortedFonts)
		for i, v in pairs(LSM:List(LSM.MediaType.FONT)) do
			tinsert(sortedFonts, v)
		end
		table.sort(sortedFonts, function(a, b) return string.upper(a) < string.upper(b) end)
		local row = 0
		local count = 0
		local s = 0
		local even = false
		for i, v in pairs(sortedFonts) do							
			local f = CreateFontFrame(self, v, even)			
			if f then
				s = s + 1
				if s == 2 then
					s = 0
					even = not even
				end
				count = count + 1
				if count % 2 ~= 0 and count > 1 then
					row = row + 1
				end
				tinsert(self.FontFrames, f)							
				f:SetParent(self.scrollchild)
				f:SetHeight(FontFrameHeight)
				if count == 1 then					
					f:SetPoint("TOPLEFT", FontFramePadding, 0)
					f:SetPoint("TOPRIGHT", self.scrollchild, "TOP", -FontFramePadding / 2, 0)
				elseif count == 2 then
					f:SetPoint("TOPLEFT", self.scrollchild, "TOP", FontFramePadding / 2, 0)
					f:SetPoint("TOPRIGHT", -FontFramePadding, 0)
				else					
					f:SetPoint("TOPLEFT", self.FontFrames[count - 2], "BOTTOMLEFT", 0, -FontFramePadding)
					f:SetPoint("TOPRIGHT", self.FontFrames[count - 2], "BOTTOMRIGHT", 0, -FontFramePadding)
				end
				
				if f.font ~= self.Font then	
					f.checked = false
				else		
					f.checked = true
					local v = row * (FontFrameHeight + FontFramePadding) - FontFramePadding
					self.scrollbar:SetValue(v)
					f.bg:SetTexture(unpack(FontFrameChecked))
				end
			end		
		end
		self.scrollchild:SetHeight(ceil(#self.FontFrames / 2) * (FontFrameHeight + FontFramePadding) - FontFramePadding)
		
		self.pullout:SetFrameStrata("FULLSCREEN_DIALOG")
		self.pullout:SetFrameLevel(this:GetFrameLevel() + 10)
		self.pullout:Show()
		self.pullout:SetToplevel(true)
		AceGUI:SetFocus(self)
	end
end 

local function Close_Pullout(self)
	for i, font in pairs(self.FontFrames) do
		font:ClearAllPoints()
		font.even = nil
		font.checked = nil
		table.insert(FontFrames, font)
	end
	wipe(self.FontFrames)
	self.pullout:Hide()
end

local function ApplySettings(self)
	self.prev.Name:SetText(self.Font)
	
	self.prev.View:SetFont(LSM:Fetch("font", self.Font), self.FontSize, self.FontFlags)
	self.prev.View:SetTextColor(unpack(self.FontColor))
end

--[[-----------------------------------------------------------------------------
Scripts
-------------------------------------------------------------------------------]]

local function Scrollbar_OnValueChanged(this, value)
	this.obj.scrollframe:SetVerticalScroll(value)
end

local function ScrollFrame_OnSizeChanged(this)
	local self = this.obj
	self.scrollchild:SetWidth(self.scrollframe:GetWidth() - 20)
	
	local ScrollMax = max(self.scrollchild:GetHeight() - self.scrollframe:GetHeight(), 0)
	self.scrollbar:SetMinMaxValues(0, ScrollMax)
end

local function ScrollFrame_OnMouseWheel(this, value)
	local self = this.obj
	local min, max = self.scrollbar:GetMinMaxValues()
	local current = self.scrollbar:GetValue()
	if IsShiftKeyDown() and value > 0 then
	   self.scrollbar:SetValue(current - (FontFrameHeight - FontFramePadding) * 2)
	elseif IsShiftKeyDown() and value < 0 then
	   self.scrollbar:SetValue(current + (FontFrameHeight + FontFramePadding) * 2)
	elseif value < 0 and current < max then
	   self.scrollbar:SetValue(current + FontFrameHeight + FontFramePadding)
	elseif value > 0 and current > 1 then
	   self.scrollbar:SetValue(current - FontFrameHeight - FontFramePadding)
	end
end

local function Sizer_OnValueChanged(this, event, value)
	self = this.obj
	self.FontSize = value
	if self.DBObject then
		self.DBObject.Size = value
	end
	self:Fire("OnFontSizeChanged", value)
	self:Fire("OnValuesChanged", value)
	ApplySettings(self)
end

local function Color_OnValueChanged(this, event, r, g, b, a)
	self = this.obj
	self.FontColor[1], self.FontColor[2], self.FontColor[3], self.FontColor[4] = r, g, b, a
	ApplySettings(self)
	if self.DBObject then
		local c = self.DBObject.Color or {}
		c[1], c[2], c[3], c[4] = r, g, b, a
		self.DBObject.Color = c
	end
	self:Fire("OnColorChanged", r, g, b, a)
	self:Fire("OnValuesChanged", value)
end

local function Flags_OnValueChanged(this, event, value)
	self = this.obj
	self.FontFlags = value
	ApplySettings(self)
	if self.DBObject then
		self.DBObject.Flags = value
	end
	self:Fire("OnFontFlagsChanged", value)
	self:Fire("OnValuesChanged", value)
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	["OnAcquire"] = function(self)
		self:SetWidth(400)
		self:SetHeight(150)
		
		local font, size, flags = GameFontNormal:GetFont()
		
		self.title:SetFont(font, size, flags)
		self.size:SetFont(font)
		self.flags:SetFont(font)
		self.flags:SetItemFont(font)
		self.color:SetFont(font)
		
		self.size.frame:Show()
		self.flags.frame:Show()
		self.color.frame:Show()
		
		
	end,
	
	["OnRelease"] = function(self)
		self.DBObject = nil
		Close_Pullout(self)
	end,
	
	["ClearFocus"] = function(self)
		Close_Pullout(self)
	end,
	
	["SetLabel"] = function(self, text)
		self.title:SetText(text)
	end,
	
	["SetFont"] = function(self, font)
		local row = 0
		self.Font = font
		ApplySettings(self)
	end,
	
	["SetHasColor"] = function(self, have)
		if have then 
			self.color.frame:Show()
		else
			self.color.frame:Hide()
		end
	end,
	
	["SetFontSize"] = function(self, size)
		self.size:SetValue(size)
		self.FontSize = size
		ApplySettings(self)
	end,
	
	["SetFontColor"] = function(self, r, g, b, a)
		if not r then return end
		if type(r) == "table" then
			self.FontColor[1], self.FontColor[2], self.FontColor[3], self.FontColor[4] = unpack(r)
			self.color:SetColor(unpack(r))
			self.FontColor = {unpack(r)}
		else
			self.FontColor[1], self.FontColor[2], self.FontColor[3], self.FontColor[4] = r, g, b, a
			self.color:SetColor(r, g, b, a)
			self.FontColor = {r, g, b, a}
		end
		
		ApplySettings(self)
	end,
	
	["SetFontFlags"] = function(self, value)
		self.FontFlags = value
		self.flags:SetValue(value)
		ApplySettings(self)
	end,
	
	["SetControlFont"] = function(self, font, size, flags)
		local _, s, f = self.title:GetFont()		
		self.title:SetFont(font, size or s, flags or f)
		self.size:SetFont(font, 14)
		self.flags:SetFont(font, 14)
		self.flags:SetItemFont(font, 14)
		self.color:SetFont(font, 14)
	end,
	
	["SetControlFontColor"] = function(self, r, g, b, a)
		if type(r) == "table" then
			r, g, b, a = unpack(r)
		end
		self.title:SetTextColor(r, g, b, a or 1)
		self.size:SetFontColor(r, g, b, a or 1)
		self.flags:SetFontColor(r, g, b, a or 1)
		self.flags:SetItemFontColor(r, g, b, a or 1)
		self.color:SetFontColor(r, g, b, a or 1)
	end,
	
	["SetControlBackgroundColor"] = function(self, r, g, b, a)
		if type(r) == "table" then
			r, g, b, a = unpack(r)
		end
		self.pane:SetBackdropColor(r, g, b, a)
	end,
	
	["SetControlBorderColor"] = function(self, r, g, b, a)
		if type(r) == "table" then
			r, g, b, a = unpack(r)
		end
		self.pane:SetBackdropBorderColor(r, g, b, a)
	end,
	
	["SetDBObject"] = function(self, obj)
		self.DBObject = obj
		self:SetFontColor(obj.Color)
		self:SetFont(obj.Font)
		self:SetFontSize(obj.Size)
		self:SetFontFlags(obj.Flags)
	end,
}
--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]

local Backdrop  = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = GetUIFPath().."\\textures\\border1.tga",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 3, right = 3, top = 3, bottom = 3 }
}


local function Constructor()	
	local count = AceGUI:GetNextWidgetNum(Type)
	local frame = CreateFrame("Frame", "UIF-Font"..count, UIParent)	
		
	local title = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	title:SetPoint("TOPLEFT", 15, -5)
	title:SetText("Font Settings")
	
	local pane = CreateFrame("Frame", nil, frame)
	pane:SetPoint("TOPLEFT", title, "BOTTOMLEFT", -15, -5)
	pane:SetPoint("BOTTOMRIGHT")
	pane:SetBackdrop(Backdrop)
	pane:SetBackdropColor(0.1, 0.1, 0.1, 0.5)
	pane:SetBackdropBorderColor(0.4, 0.4, 0.4)
			
	local prev = CreateFrame("Frame", nil, pane)
	prev:SetPoint("TOPLEFT", 10, -10)
	prev:SetPoint("BOTTOMRIGHT", pane, "CENTER", -5, 0)	
	Backdrop.edgeFile = GetUIFPath().."\\textures\\border3.tga"
	prev:SetBackdrop(Backdrop)
	prev:SetBackdropColor(0, 0, 0, 0.5)
	prev:SetBackdropBorderColor(0.4, 0.4, 0.4)
	
	local button = CreateFrame("Frame", nil, prev)
	button:SetPoint("TOPRIGHT", -3, -3)
	button:SetPoint("BOTTOM", 0, 3)
	button:SetWidth(10)
	button:SetScript("OnMouseDown", FontFrame_Show)
	button:SetScript("OnEnter", function(self)
		self.texture:SetTexture(0.2, 0.2, 0.6, 1)
	end)
	button:SetScript("OnLeave", function(self)
		self.texture:SetTexture(0, 0.5, 0.5, 1)
	end)
	local t = button:CreateTexture(nil, "ARTWROK")
	t:SetTexture(0, 0.5, 0.5, 1)
	t:SetAllPoints()
	button.texture = t
	
	local fontName = prev:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	fontName:SetPoint("BOTTOMLEFT", 3, 3)
	fontName:SetPoint("RIGHT", button, "LEFT")
	fontName:SetHeight(20)
	fontName:SetText("GameFontNormal")
	prev.FontName = fontName
	
	local fontView = prev:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	fontView:SetPoint("TOPLEFT", 3, -3)
	fontView:SetPoint("BOTTOMRIGHT", fontName, "TOPRIGHT")
	fontView:SetText("123456ABCabc![\\//]")
	
	prev.Name = fontName
	prev.View = fontView
	
	local pullout = CreateFrame("Frame", nil, UIParent)
	Backdrop.edgeFile = GetUIFPath().."\\textures\\border1.tga"
	pullout:SetBackdrop(Backdrop)
	pullout:SetBackdropColor(0, 0, 0, 1)
	pullout:SetBackdropBorderColor(0.4, 0.4, 0.4, 1)
	pullout:Hide()
	
	-- Scrollchild
	local scrollchild = CreateFrame("Frame", "UIF-FontScrollChild"..count, pullout)
	
	-- Scrollframe
	local scrollframe = CreateFrame("ScrollFrame", "UIF-FontScrollFrame"..count, pullout)
	scrollframe:SetPoint("TOPLEFT", 4, -2 - 5)
	scrollframe:SetPoint("BOTTOMRIGHT", -2, 2 + 5)
	scrollframe:SetScrollChild(scrollchild)
	scrollframe:SetScript("OnSizeChanged", ScrollFrame_OnSizeChanged)
	scrollframe:SetScript("OnMouseWheel", ScrollFrame_OnMouseWheel)
	
	-- Scrollbar
	local scrollbar = CreateFrame("Slider", "UIF-FontScrollBar"..count, scrollframe, "UIPanelScrollBarTemplate")
	scrollbar:SetPoint("TOPRIGHT", scrollframe, -5, -18)
	scrollbar:SetPoint("BOTTOMRIGHT", scrollframe, -5, 18)
	scrollbar:SetMinMaxValues(0, 1000)
	scrollbar:SetValueStep(1)
	scrollbar:SetValue(0)
	scrollbar:SetWidth(10)
	scrollbar:SetScript("OnValueChanged", Scrollbar_OnValueChanged)
		
	local scrollbg = scrollbar:CreateTexture(nil, "BACKGROUND")
	scrollbg:SetAllPoints(scrollbar)
	scrollbg:SetTexture(0.4, 0.4, 0.4, 1)
	
	
	-- Size
	local sizer = AceGUI:Create("UIF-Slider")
	sizer.frame:SetParent(frame)	
	sizer.frame:SetPoint("TOPLEFT", pane, "TOP", 10, -10)
	sizer.frame:SetPoint("TOPRIGHT", -10, -10)
	sizer:SetLabel("Size")
	sizer:SetSliderValues(1, 40, 1)
	sizer:SetCallback("OnValueChanged", Sizer_OnValueChanged)
	sizer.frame:Show()
	
	-- Flags
	local flags = AceGUI:Create("UIF-Dropdown")
	flags.frame:SetParent(pane)
	flags.frame:SetPoint("BOTTOMLEFT", 10, 10)
	flags.frame:SetPoint("RIGHT", pane, "CENTER")
	flags:SetLabel("Outline")
	flags:SetList(FontFlags)
	flags:SetCallback("OnValueChanged", Flags_OnValueChanged)
	
	-- Color
	local color = AceGUI:Create("UIF-ColorPicker")
	color.frame:SetParent(frame)
	color.frame:SetPoint("BOTTOMLEFT", pane, "BOTTOM", 10, 10)
	color.frame:SetPoint("RIGHT", pane, -10, 10)
	color:SetLabel("Color")
	color.frame:Show()
	color:SetHasAlpha(true)
	color:SetCallback("OnValueChanged", Color_OnValueChanged)
	color:SetCallback("OnValueConfirmed", Color_OnValueChanged)
	
	local widget = {
		FontFrames = {},
		title = title,
		frame = frame,
		pane = pane,
		prev = prev,
		pullout = pullout,
		scrollframe = scrollframe,
		scrollchild = scrollchild,
		scrollbar = scrollbar,
		flags = flags,
		color = color,
		size = sizer,
		type = Type,
		FontSize = 12,
		FontFlags = "none",
		FontColor = {1, 1, 1, 1},
	}
	
	for method, func in pairs(methods) do
		widget[method] = func
	end
	
	for i, v in pairs({flags, frame, sizer, color, button, scrollframe, scrollchild, scrollbar}) do
		v.obj = widget
	end	
	
	return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)