local mod = RW:Boss("Chimaeron", "Blackwing Descent")
local L

mod.BossIcon = [[Interface\ICONS\achievement_dungeon_blackwingdescent_raid_chimaron]]

---------------------------
-- Variables
-- Break
local tBreakCD, mBreak, tBreakDur

-- Double attack
local tDoubleAttackCD, wDoubleAttack

-- Feud
local tFeudDur, wFeud

-- Reroute
local tRerouteDur, wReroute

local tBerserk
---------------------------
-- Init

function mod:OnRegister()
	L = mod:RegisterLocale(self:GetName(), "enUS", true)
	if L then
		L["%s Soon!"] = true
		L["Warning for Mortality soon"] = true
	end
	L = mod:GetLocale()
	self.L = L
	
	self:SetTrigger(43296)
	self:SetWin()
	
	-- Break
	tBreakCD		= self:CDTimer(15, 82881)
	tBreakDur		= self:DurTargetStackTimer(60, 82881)
	mBreak			= self:TargetStackMessage(82881)
	
	-- Double Attac
	tDoubleAttackDur = self:DurTimer(5, 88826)
	wDoubleAttack	= self:CastWarning(88826)
	
	-- Feud
	wFeud			= self:CastWarning(88872)
	tFeudDur		= self:DurTimer(26, 88872)
	
	-- Massacre
	self:RegisterAlert("tMasacreCD", self:CDTimer(30, 82848))
	self:RegisterAlert("tMasacreCast", self:CastTimer(4, 82848))
	self:RegisterAlert("wMassacre", self:CastWarning(82848))
	
	-- Mortality
	self:RegisterAlert("wImortality", self:CustomWarning(L["%s Soon!"], L["Warning for Mortality soon"], 82890))
	self:RegisterAlert("wImortalityCast", self:CastWarning(82890))
	
	-- Caustic Slime
	self:RegisterAlert("iSlime", self:Icon("STAR", 88915, false, 3, "CIRCLE", "DIAMOND", "TRIANGLE", "SQUARE"))
	
	tBerserk 		= self:BerserkTimer(450)
end

---------------------------
-- Locals
local gainSpam
local ImortalityWarned
local BreakCount
local FeudActive
local MortalityCasted

---------------------------
-- Start

function mod:OnEngage(trigger)
	self:RegisterEvent("UNIT_HEALTH")

	self:Track("Break", 82881)
	self:Track("DoubleAttack", 88826)
	self:Track("Feud", 88872)
	self:Track("Massacre", 82848)
	self:Track("Slime", 88915, 82935, 88916, 88917)
	self:Track("Mortality", 82890)
	
	RW.Callbacks:Fire("RangeCheckShow", true, 6)
	
	gainSpam = 0
	ImortalityWarned = false
	BreakCount = 0
	FeudActive = false
	MortalityCasted = false
	
	self:Call("tMasacreCD", "Start")
	
	if self:IsHeroic() then
		tBerserk:Start()
	end
end

function mod:Break(event, args)
	if event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_APPLIED_DOSE" then
		if self:InstanceSize() == 25 then
			if BreakCount == 1 then
				tBreakCD:Start(15)
			else
				tBreakCD:Start()
			end
		else
			tBreakCD:Start(20)
		end
		tBreakDur:Start(args.dName, args.amount or 1)
		mBreak:Show(args.dName, args.amount or 1)
		
		BreakCount = BreakCount + 1
	end
end

function mod:DoubleAttack(event, args)
	if event == "SPELL_AURA_APPLIED" then
		tDoubleAttackDur:Start()
		wDoubleAttack:Show()
	end
end

local function InactivateFeud()
	FeudActive = false
end

function mod:Feud(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		wFeud:Show()
		tFeudDur:Start()
		FeudActive = true
		self:Schedule("InactivateFeud", 25, InactivateFeud)
	end
end

function mod:Massacre(event, args)
	if event == "SPELL_CAST_START" then
		self:Call("tMasacreCD", "Start")
		self:Call("tMasacreCast", "Start")
		self:Call("wMassacre", "Show")
	end
end

function mod:Slime(event, args)
	if event == "SPELL_AURA_APPLIED" and not FeudActive then
		self:Call("iSlime", "Show", args.dName)
	
	elseif event == "SPELL_AURA_REMOVED" then
		--self:Call("iSlime", "Hide", args.dName)
	end
end

function mod:Mortality(event, args)
	if event == "SPELL_CAST_SUCCESS" and not MortalityCasted then
		MortalityCasted = true
		self:Call("wImortalityCast", "Show")
	end
end

function mod:UNIT_HEALTH(event, unit)
	if unit == "boss1" then
		local hp = UnitHealth(unit) / UnitHealthMax(unit) * 100
		if hp <= 25 and not ImortalityWarned then
			ImortalityWarned = true
			self:Call("wImortality", "Show")
		end
	end
end