local mod = RW:Boss("Nefarian", "Blackwing Descent")
local L

mod.BossIcon = [[Interface\ICONS\achievement_dungeon_blackwingdescent_raid_nefarian]]

---------------------------
-- Variables

---------------------------
-- Init

function mod:OnRegister()
	L = mod:RegisterLocale(self:GetName(), "enUS", true)
	if L then
		L["Phase 2 Emote"] = "Curse you, mortals! Such a callous disregard for one's possessions must be met with extreme force!"
		L["Phase 3 Emote"] = "I have tried to be an accommodating host, but you simply will not die! Time to throw all pretense aside and just... KILL YOU ALL!"
		
		L["Phase Change"] = "Phase Change"
		
		L["Electrocute Emote"] = "The air crackles with electricity!"
		
		L["Stolen Power x %d"] = true
	end
	L = mod:GetLocale()
	self.L = L
	
	self:SetTrigger(41376, 41270) -- Nefarian, Onyxia
	self:SetWin()
	
	self:RegisterAlert("mPhase", self:PhaseMessage())
	
	self:RegisterAlert("tPhaseChange", self:CustomTimer(180, L["Phase Change"], L["Phase Change"], [[Interface\Icons\Achievement_BG_killingblow_30]]))
	
	-- Electrocute
	self:RegisterAlert("wElectrocute", self:CastWarning(94088))
	self:RegisterAlert("tElectrocute", self:CastTimer(5, 94088))
	
	-- Cinders
	self:RegisterAlert("iCinders", self:Icon("STAR", 79339, false, 8, "CIRCLE", "TRIANGLE"))
	self:RegisterAlert("mCinders", self:TargetMessage(79339))
	self:RegisterAlert("tCindersDur", self:DurTimer(8, 79339))
	self:RegisterAlert("wCinders", self:RunawayWarning(79339))
	self:RegisterAlert("wCindersCast", self:CastWarning(79339, nil, nil, false))
	self:RegisterAlert("sCinders", self:SayOnPlayer(79339))
	
	-- Shadow Blaze
	self:RegisterAlert("tBlazeCD", self:CDTimer(35, 94085))
	
	-- Stolen Power
	self:RegisterAlert("tStolenPower", self:DurTargetStackTimer(18, 80627))
	self:RegisterAlert("wStolenPower", self:CustomWarning(L["Stolen Power x %d"], "Max Stacks", 80627))
	
	-- Dominion
	self:RegisterAlert("wDominion", self:OnYouWarning(79318))
end

---------------------------
-- Locals
local CindersSpell = GetSpellInfo(79339)
local BlazeTimer

---------------------------
-- Start

function mod:OnEngage(trigger)
	self:Track("Electrocute", 94088, 82172, 94089, 94090)
	self:Track("Cinder", 79339)
	self:Track("StolenPower", 80627)
	self:Track("Dominion", 79318)

	self:YellTrack("Phase2", L["Phase 2 Emote"])
	self:YellTrack("Phase3", L["Phase 3 Emote"])
	self:YellTrack("ElectrocuteEmote", L["Electrocute Emote"])
	
	RW.Callbacks:Fire("MPBPowerByID", 41270, "Default", nil, nil, true)
	
	BlazeTimer = 35
end

function mod:Electrocute(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		if args:IsPlayer() then
		end
	end
end

function mod:Phase2(event, msg, name)
	self:Call("mPhase", "Show", 2)
	self:Call("tPhaseChange", "Start")
	RW.Callbacks:Fire("MPBPowerHideByID", 41270)
end

function mod:BlazeCDFunction()
	if BlazeTimer > 10 then
		BlazeTimer = BlazeTimer - 5
	end
	
	self:Call("tBlazeCD", "Start", BlazeTimer)
	self:Schedule("BlazeCDSchedule", BlazeTimer, "BlazeCDFunction")
end

function mod:Phase3(event, msg, name)
	self:Call("tBlazeCD", "Start", 10)
	self:Schedule("BlazeCDSchedule", 10, "BlazeCDFunction")
	self:Call("mPhase", "Show", 3)
end

function mod:ElectrocuteEmote(event, msg, name)
	self:Call("wElectrocute", "Show")
	self:Call("tElectrocute", "Start")
end

function mod:Cinder(event, args)
	if event == "SPELL_AURA_APPLIED" then
		self:Call("iCinders", "Show", args.dName)
		self:Call("mCinders", "AddDelayTarget", args.dName)
		self:Call("mCinders", "DelayedShow", 0.1)
		self:Call("tCindersDur", "Start")
		
		if args:IsPlayer() then
			self:Call("wCinders", "Show")
			self:Call("sCinders", "Show", args.dName)
			RW.Callbacks:Fire("RangeCheckShow", true, 10)
		end
	
	elseif event == "SPELL_AURA_REMOVED" then
		self:Call("iCinders", "Hide", args.dName)
		if args:IsPlayer() then
			RW.Callbacks:Fire("RangeCheckHide")
		end
	
	elseif event == "SPELL_CAST_START" then
		self:Call("wCindersCast", "Show")
	end
end

function mod:StolenPower(event, args)
	if event == "SPELL_AURA_APPLIED_DOSE" then
		if args:IsPlayer() then			
			self:Call("tStolenPower", "Start", args.dName, args.amount)
			if (args.amount or 0) >= 150 then
				self:Call("wStolenPower", "Show", args.amount)
			end
		end
	end
end

function mod:Dominion(event, args)
	if event == "SPELL_AURA_APPLIED" then
		if args:IsPlayer() then
			self:Call("wDominion", "Show")
		end
	end
end