local mod = RW:Boss("Sinestra", "The Bastion of Twilight")
local L

mod.BossIcon = [[Interface\ICONS\Achievement_Dungeon_Bastion of Twilight_LadySinestra]]

---------------------------
-- Variables

---------------------------
-- Init

function mod:OnRegister()
	L = mod:RegisterLocale(self:GetName(), "enUS", true)
	if L then
		L.EmoteWeakenedEgg = "You mistake this for weakness?  Fool!"
		L.EmoteWhelps = "Feed, children!  Take your fill from their meaty husks!"
		
		L.Whelps = true
		L["Weakened Egg"] = true
		L["Weakened Egg Soon"] = true 
	end
	L = mod:GetLocale()
	self.L = L
	
	self:SetTrigger(45213)
	self:SetWin()
	
	
	-- Flame Breath
	self:RegisterAlert("tBreathCD", self:CDTimer(24, 92944))
	self:RegisterAlert("wBreath", self:CastWarning(92944))
	
	-- Twilight Extinction
	self:RegisterAlert("tExtinction", self:CastTimer(15, 86227))
	self:RegisterAlert("wExtinction", self:CastWarning(86227))
	
	-- Twilight Slicer
	self:RegisterAlert("tSlicerCD", self:CDTimer(28, 92954))
	self:RegisterAlert("wSlicer", self:Warning(92954))
	
	self:RegisterAlert("tWhelpCD", self:CDTimer(50, L.Whelps, 10697, L.Whelps))
	self:RegisterAlert("mWhelps", self:Message(L.Whelps, 10697, L.Whelps))
	
	self:RegisterAlert("tWeakenedEgg", self:Timer(5, L["Weakened Egg"], [[Interface\ICONS\inv_egg_02]], L["Weakened Egg"]))
	self:RegisterAlert("tWeakenedEggDur", self:Timer(30, L["Weakened Egg"], [[Interface\ICONS\inv_egg_02]], L["Weakened Egg"]))
	self:RegisterAlert("wWeakenedEgg", self:Warning(L["Weakened Egg Soon"], [[Interface\ICONS\inv_egg_02]], L["Weakened Egg"]))
	self:RegisterAlert("mWeakenedEggS", self:Message(L["Weakened Egg"], [[Interface\ICONS\inv_egg_02]], L["Weakened Egg"]))
	
	
	self:RegisterAlert("tEssence", self:DurTimer(180, 87946))
	
	self:RegisterAlert("mPhase", self:PhaseMessage())
end

---------------------------
-- Locals
local DeadEggs
local SlicerTimer

SlicerTimer = function()
	mod:Call("wSlicer", "Show")
	mod:Call("tSlicerCD", "Start")
	mod:Schedule("SlicerTimer", 28, SlicerTimer)
end

---------------------------
-- Start

function mod:OnEngage(trigger)
	self:Track("Breath", 92944)
	self:Track("Extinction", 86227)
	self:Track("WeakenedEgg", 87654)
	self:Track("Essence", 87946)
	
	self:YellTrack("WeakenedEggEmote", L.EmoteWeakenedEgg)
	self:YellTrack("Whelps", L.EmoteWhelps)
	
	self:RegisterEvent("UNIT_HEALTH")
	
	DeadEggs = 0
	
	self:Call("tWhelpCD", "Start", 16)
	self:Call("tSlicerCD", "Start", 30)
	self:Schedule("SlicerTimer", 30, SlicerTimer)
end

function mod:Breath(event, args)
	if event == "SPELL_CAST_START" then
		self:Call("tBreathCD", "Start")
		self:Call("wBreath", "Show")
	end
end

function mod:Extinction(event, args)
	if event == "SPELL_CAST_START" then
		self:Call("tExtinction", "Start")
		self:Call("wExtinction", "Show")
		self:Call("tSlicerCD", "Stop")
		self:Unschedule("SlicerTimer")
	end
end

function mod:WeakenedEgg(event, args)
	if event == "SPELL_AURA_REMOVED" then
		self:Call("mWeakenedEggS", "Show")
		self:Call("tWeakenedEggDur", "Start")
	end
end

function mod:Essence(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		self:Call("tEssence", "Start")
	end
end

function mod:WeakenedEggEmote(event, msg, name)
	self:Call("tWeakenedEgg", "Start")
	self:Call("wWeakenedEgg", "Show")
end

function mod:Whelps(event, msg, name)
	self:Call("tWhelpCD", "Start")
	self:Call("mWhelps", "Show")		
end

function mod:UNIT_HEALTH(unit)
	if unit == "boss1" then
		local hp = UnitHealth(unit) / UnitHealthMax(unit) * 100
		if hp < 31 then
			self:Call("mPhase", "Show", 2)
			self:Call("tBreathCD", "Stop")
			self:Call("tSlicerCD", "Stop")
			self:Unschedule("SlicerTimer")
			self:UnregisterEvent("UNIT_HEALTH")
			self:RegisterEvent("UNIT_DIED")
		end
	end
end

function mod:UNIT_DIED(cID, gUID)
	if cID == 46842 then
		DeadEggs = DeadEggs + 1
		if DeadEggs == 2 then
			self:Call("tWeakenedEggDur", "Stop")
			self:Call("mPhase", "Show", 3)
			self:Call("tBreathCD", "Start")
			self:Call("tSlicerCD", "Start", 29)
			self:Schedule("SlicerTimer", 29, SlicerTimer)
		end
	end
end