local mod = RW:Boss("Valiona and Theralion", "The Bastion of Twilight")
local L
local LBB

mod.BossIcon = [[Interface\ICONS\Achievement_Dungeon_Bastion of Twilight_Valiona Theralion]] 

-- Valiona  45992
-- Theralion 45993

---------------------------
-- Variables

---------------------------
-- Init

function mod:OnRegister()
	L = mod:RegisterLocale(self:GetName(), "enUS", true)
	if L then
		L["Deep Breath"] = "Theralion, I will engulf the hallway. Cover their escape!"
		
		L["Dragon Landing"] = "%s landing"
		L["Phase switches"] = true
	end
	L = mod:GetLocale()
	self.L = L
	
	LBB = RW.LBB
	
	self:SetTrigger(45992, 45993)
	self:SetWin()
	
	self:AddToCategory(LBB["Theralion"], 86408, 95639, 88518)
	self:AddToCategory(LBB["Valiona"], 86788, 86840, 86059)
	
	self:RegisterAlert("tSwitch", self:CustomTimer(100,  L["Dragon Landing"], L["Phase switches"], nil, L["Phase switches"]))
	
	-- Valiona
		-- Blackout
		self:RegisterAlert("iBlackout", self:Icon("SKULL", 86788, false, 15))
		self:RegisterAlert("wBlackoutSelf", self:OnYouWarning(86788))
		self:RegisterAlert("wBlackout", self:CastWarning(86788))
		self:RegisterAlert("tBlackoutDur", self:DurTargetTimer(15, 86788))
		self:RegisterAlert("tBlackoutCD", self:CDTimer(45, 86788))
		self:RegisterAlert("mBlackout", self:TargetMessage(86788))
		
		-- Devouring Flames
		self:RegisterAlert("wFlames", self:RunawayWarning(86840))
		self:RegisterAlert("tFlamesCast", self:Timer(2.5, 86840))
		self:RegisterAlert("tFlamesCD", self:CDTimer(45, 86840))
		
		-- Deep Breath
		self:RegisterAlert("tDeepBreathCD", self:CDTimer(15, 86059))
		self:RegisterAlert("wDeepBreath", self:RunawayWarning(86059))
		
	-- Theralion
		-- Engulfing Magic
		self:RegisterAlert("iEngulf", self:Icon("SKULL", 95639, false, 20, "CROSS", "CIRCLE"))
		self:RegisterAlert("wEngulf", self:RunawayWarning(95639))
		self:RegisterAlert("mEngulf", self:TargetMessage(95639))
		
		-- Twilight Meteorite
		self:RegisterAlert("wMeteorite", self:OnYouWarning(88518))
		self:RegisterAlert("iMeteorite", self:Icon("STAR", 88518, false, 6, "TRIANGLE"))
		
		-- Dazzling Destruction
		self:RegisterAlert("wDazzling", self:RunawayWarning(86408))
		self:RegisterAlert("tDazzlingDur", self:DurTimer(15, 86408))
		self:RegisterAlert("tDazzelingCD", self:CDTimer(131, 86408))
		
		-- Twilight Blast
		self:RegisterAlert("wBlast", self:RunawayWarning(92900))
		self:RegisterAlert("sBlast", self:SayOnPlayer(92900))
		
	-- Twilight Shift
	self:RegisterAlert("tShiftDur", self:DurTargetStackTimer(120, 93051))
	self:RegisterAlert("tShiftCD", self:CDTimer(20, 93051))
	self:RegisterAlert("mShift", self:TargetStackMessage(93051))
end

---------------------------
-- Locals
local DazzlingWarned = 0
local DeepBreathWarned = 0
local DeepBreathCount = 0
local MeteoriteWarned = {}

local EngulfSpell = GetSpellInfo(95639)
local BlackoutSpell = GetSpellInfo(86788)
local MeteoriteSpell = GetSpellInfo(88518)
local BlastSpell = GetSpellInfo(92900)

---------------------------
-- Start

function mod:OnEngage(trigger)
	self:RegisterEvent("UNIT_AURA")
	self:Track("Blackout", 86788, 92878, 92877, 92876)
	self:Track("Flames", 86840)
	self:YellTrack("DeepBreath", L["Deep Breath"])

	self:Track("Engulf", 95639, 86622, 95641, 95640)
	self:Track("TwilightBlast", 92900, 95416, 92898, 86369, 92899)
	self:Track("Dazzling", 86408)
	
	self:Track("TwilightShift", 93051)
	
	self:Call("tSwitch", "Start", LBB["Theralion"])
	self:Call("tFlamesCD", "Start", 25)
	self:Call("tDazzelingCD", "Start", 85)
	
	DazzlingWarned = 0
	DeepBreathWarned = 0
	DeepBreathCount = 0
	wipe(MeteoriteWarned)
end

-- Valiona
do
	function mod:Blackout(event, args)
		if event == "SPELL_AURA_APPLIED" then
			self:Call("tBlackoutCD", "Start")
			self:Call("tBlackoutDur", "Start", args.dName)
			self:Call("iBlackout", "Show", args.dName)
			self:Call("mBlackout", "Show", args.dName)
			
			if args:IsPlayer() then
				self:Call("wBlackoutSelf", "Show")
				SendChatMessage(BlackoutSpell.." on "..self.pName.."!", "SAY")
			else
				self:Call("wBlackout", "Show")
			end
			
		elseif event == "SPELL_AURA_REMOVED" then
			self:Call("iBlackout", "Hide", args.dName)
			self:Call("tBlackoutDur", "Stop", args.dName)
		end
	end

	function mod:Flames(event, args)
		if event == "SPELL_CAST_START" then
			self:Call("wFlames", "Show")
			self:Call("tFlamesCast", "Start")
			self:Call("tFlamesCD", "Start")
		end
	end

	local function WarnDeppBreath()
		DeepBreathCount = DeepBreathCount + 1
		mod:Call("tDeepBreathCD", "Start")
		mod:Call("wDeepBreath", "Show")
		if DeepBreathCount < 2 then
			mod:Schedule("WarnDeppBreath", 15, WarnDeppBreath)		
		end
	end

	function mod:DeepBreath(event, msg, name)
		self:Call("tSwitch", "Start", 50, LBB["Valiona"])
		self:Call("tSwitch", "Schedule", 50, 120, LBB["Theralion"])
		self:Call("tDeepBreathCD", "Start", 15)
		self:Call("wDeepBreath", "Show")
		self:Call("tDazzelingCD", "Start")
		DeepBreathCount = 1
		self:Schedule("WarnDeppBreath", 15, WarnDeppBreath)
		
		-- Cleanup
	end
end

-- Theralion
do
	function mod:Engulf(event, args)
		if event == "SPELL_AURA_APPLIED" then
			self:Call("iEngulf", "Show", args.dName)
			self:Call("mEngulf", "AddDelayTarget", args.dName)
			self:Call("mEngulf", "DelayedShow", 0.2)
			
			if args:IsPlayer() then
				self:Call("wEngulf", "Show")
				SendChatMessage(EngulfSpell.. " on "..self.pName.."!", "SAY")
			end
		
		elseif event == "SPELL_AURA_REMOVED" then
			self:Call("iEngulf", "Hide", args.dName)
		end
	end
	
	function mod:Dazzling(event, args)
		if event == "SPELL_CAST_START" and GetTime() - DazzlingWarned > 30 then
			DazzlingWarned = GetTime()
			self:Call("wDazzling", "Show")
			self:Call("tDazzlingDur", "Start")
			self:Call("tSwitch", "Start", 15, LBB["Theralion"])
			
			-- Deep breath start 1:35 duration 0:50
			self:Call("tDeepBreathCD", "Schedule", 15, 95)
			self:Call("tSwitch", "Schedule", 15.1, 145, LBB["Valiona"])
			
			-- Cleanup
			self:Call("tFlamesCD", "Stop")
			self:Call("tBlackoutCD", "Stop")
		end
	end
	
	local function BlastTarget(sGUID)
		local target = UnitGUID("boss1") == sGUID and "boss1target" or "boss2target"
		if target and UnitName(target) then
			if UnitIsUnit(target, "player") then
				mod:Call("wBlast", "Show")
				mod:Call("sBlast", "Show", mod.pName)
			end
		end
		
	end
	
	function mod:TwilightBlast(event, args)
		if event == "SPELL_CAST_START" then
			self:Schedule("BlastTarget", 0.3, BlastTarget, args.sGUID)
		end
	end
	
	function mod:UNIT_AURA(event, uID)
		local uName = UnitName(uID)
		local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitDebuff(uID, MeteoriteSpell)
		if name and not MeteoriteWarned[uName] then
			self:Call("iMeteorite", "Show", uName)
			
			if uName == self.pName then
				self:Call("wMeteorite", "Show")
			end
			MeteoriteWarned[uName] = GetTime()
			
		elseif not name and MeteoriteWarned[uName] then
			self:Call("iMeteorite", "Hide", uName)			
			MeteoriteWarned[uName] = nil
		end
	end
end

function mod:TwilightShift(event, args)
	if event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_APPLIED_DOSE" then
		self:Call("tShiftDur", "Start", args.dName, args.amount or 1)
		self:Call("tShiftCD", "Start")
		self:Call("mShift", "Show", args.dName, args.amount or 1)
	end
end
