--[[
Plugin for showing health frames
]]

----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("MultiPurposeBars")
if not plugin then return end

RWDGB_HEALTHBARS2 = plugin

---------------------------------
-- Locals

local LSM = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-MultiPurposeBars")
local LB = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")
local AceGUI = LibStub("AceGUI-3.0")
local FA = LibStub("LibFrameAnchorRegistry-1.0")
local UIF
local Party = RW.Party
local Utils = RW.Utils

local db

local BlizzBossAnchor = CreateFrame("Frame")

local MarkerCache = {}
local BarCache = {}
local AllBars = {}
local Anchors = {}
local DefaultAnchor

local dragStart, dragStop
local backdrop = {}

local iconCoords = {
	{0, 	0.25,	0, 		0.25},
	{0.25,	0.5,	0,		0.25},
	{0.5, 	0.75,	0,		0.25},
	{0.75,	1,		0,		0.25},
	{0,		0.25,	0.25,	0.5},
	{0.25,	0.5,	0.25,	0.5},
	{0.5,	0.75,	0.25,	0.5},
	{0.75,	1,		0.25,	0.5},
}

local defaults = {
	profile = {
		PluginEnabled = true,
		
		HideBlizzFrames = false,
		
		Anchors = {
			["**"] = {
				Position = {0 , 0},
				AnchorFrame = nil,
				Point = "CENTER",
				RelPoint = "CENTER",
				
				Alpha = 1,
				Scale = 1,
				UseModel = true,
				Camera = 0,
				
				BorderColor = {0.15, 0.24, 0.31, 1},
				BorderWidth = 12,
				BorderTexture = "IshBorder",	
				StatusbarTexture = "HalD",
				StatusbarColor = {0.22, 0.34, 0.46, 1},
											
				BackgroundColor = {0, 0, 0, 0.5},
				
				Width = 280,
				Height = 45,				
				
				FontSettings = {
					Font = "accid",
					Size = 16,
					Flags = "OUTLINE",
					Color = {0.47, 0.75, 1, 1},
				},
				
				-- Markers
				MarkerWidth = 2.5,
				MarkerColor = {1, 0, 0, 0.5},
				ShowMarkers = true,
				
				-- Layout
				VerticalSpacing = 0,
				Inset = 4,
				ShowIcon = true,
				IconsSide = "LEFT",
				PercentSide = "LEFT",
				GrowUp = true,
				
				-- Formatting
				FormatString = "%C / %T (%P%)"
			},
			
			Default = {
			},
			["Health Frames"] = {
			}
		}
	}
}


-- ~~~~~~~~~~~~~~~~~~~~~~~~
-- Init ~~~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("MultiPurposeBars", defaults)	
	db = self.db.profile
	
	-- DB Upgrade
	for name, profile in pairs(self.db.profiles) do
		for i, anchor in pairs(profile.Anchors) do
			if i~= "**" then
				local db = anchor
				if db.FontSize or db.Font or db.FontFlags then
					if not db.FontSettings then db.FontSettings = {} end
					db.FontSettings.Font = db.Font
					db.FontSettings.Size = db.FontSize
					db.FontSettings.Flags = db.FontFlags
					db.FontSettings.Color = db.FontColor
					db.Font = nil
					db.FontSize = nil
					db.FontFlags = nil
					db.FontColor = nil
				end
			end
		end		
	end	
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
end

function plugin:PluginEnable()
	self:Reset()
	if not db.PluginEnabled then
		self:Disable()
		return
	end
	
	self:RegisterCallback("ConfigMode")
	
	self:RegisterCallback("MPBHealthByID")
	self:RegisterCallback("MPBPowerByID")		
	self:RegisterCallback("MPBSpellStackAura")
	self:RegisterCallback("MPBSpellStackCombat")	
	self:RegisterCallback("MPBExternal")
		
	self:RegisterCallback("MPBHealthHideByID")
	self:RegisterCallback("MPBPowerHideByID")
	self:RegisterCallback("MPBSpellStackHide")
	self:RegisterCallback("MPBExternalHide")
	self:RegisterCallback("MPBHideAll")
end

function plugin:PluginDisable()
	self:UnregisterAllCallbacks()
end

function plugin:Reset(event)
	if event and event == "OnProfileReset" then
		for i, d in pairs(db.Anchors) do
			if i ~= "**"  then
				db.Anchors[i] = nil
			end
		end
		local t = db.Anchors.Default
		t = db.Anchors["Health Frames"]
	end

	db = self.db.profile
	RWDGB_HB_DB = db
	
	for anchorName, anchor in pairs(Anchors) do		
		anchor:Delete()
		Anchors[anchorName] = nil
	end
	
	for anchorName, d in pairs(db.Anchors) do
		if anchorName ~= "**" then
			self:CreateAnchor(anchorName)
		end
	end
	
	DefaultAnchor = Anchors["Default"]
	
	self:ApplySettings()
	
	if db.HideBlizzFrames then
		for i=1, 4 do 
			_G["Boss"..i.."TargetFrame"]:SetParent(BlizzBossAnchor)
		end
		BlizzBossAnchor:Hide()
	else
		for i=1, 4 do 
			_G["Boss"..i.."TargetFrame"]:SetParent(UIParent)
		end
	end
end

function plugin:ApplySettings()
	for i, anchor in pairs(Anchors) do
		anchor:ApplySettings()
	end
end

function plugin:GetAnchor(name, preferedDefault)	
	local anchor
	if name then anchor = Anchors[name] end
	if not anchor and preferedDefault then anchor = Anchors[preferedDefault] end
	if not anchor then anchor = DefaultAnchor end
	return anchor
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~
-- Callbacks ~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~
local function MPBIDCreate(bType, id, anchorID, markers, icon, alternate)
	if not id then return end
	local anchor = plugin:GetAnchor(anchorID, "Health Frames")
	
	for i, bar in pairs(anchor.bars) do
		if bar.typeData == id and bar.type == bType and not bar.stopping then return end
	end
	
	local bar = plugin:CreateBar()
	bar.type = bType
	bar.typeData = id
	
	if icon then
		bar.icon:SetTexture(icon)
		bar.IconOverride = true
	end
	
	if markers and type(markers) == "table" then
		plugin:HandleMarkers(anchor.db, bar, markers)
	end
	
	if alternate then
		bar.alternate = true
	end
	
	anchor:AddBar(bar)
end

function plugin:MPBHealthByID(event, id, anchorID, markers, icon)
	MPBIDCreate("HealthID", id, anchorID, markers, icon)
end

function plugin:MPBPowerByID(event, id, anchorID, markers, icon, alternate)
	MPBIDCreate("PowerID", id, anchorID, markers, icon, alternate)
end

function plugin:MPBHealthHideByID(event, id)
	for anchorName, anchor in pairs(Anchors) do
		for i, bar in pairs(anchor.bars) do
			if bar.typeData == id and bar.type:sub(1, 6) == "Health" then
				bar:Hide()
			end
		end
	end
end

function plugin:MPBPowerHideByID(event, id)
	for anchorName, anchor in pairs(Anchors) do
		for i, bar in pairs(anchor.bars) do
			if bar.typeData == id and bar.type:sub(1, 5) == "Power" then
				bar:Hide()
			end
		end
	end
end

function plugin:MPBSpellStackAura(event, id, spell, maxStacks, anchorID, buffType, markers, icon)
	if not id or not spell then return end
	local anchor = plugin:GetAnchor(anchorID)
	
	for i, bar in pairs(anchor.bars) do
		if bar.typeData == id and bar.spell == spell and bar.type == "SpellStack" then return end
	end
	
	local bar = self:CreateBar()
	bar.type = "SpellStackAura"
	bar.spell = spell
	local spellName, rank, icon = GetSpellInfo(spell)
	bar.spellName = spellName
	bar.spellIcon = icon
	bar.maxStacks = maxStacks or 20
	bar.typeData = id
	bar.icon:SetTexture(icon)
	
	bar.strName:SetText(spellName)
	bar.strBottom:SetText("0 / "..bar.maxStack)
	bar.bar:SetValue(0)
	
	if markers and type(markers) == "table" then
		plugin:HandleMarkers(anchor.db, bar, markers)
	end
	
	anchor:AddBar(bar)
	
	bar:Show()
end

function plugin:MPBSpellStackCombat(event, id, spells, maxStacks, anchorID, markers, icon)
	if not id or not spells then return end
	local anchor = plugin:GetAnchor(anchorID)
	
	for i, bar in pairs(anchor.bars) do
		if bar.typeData == id and bar.type == "SpellStackCombat" then
			for i, spell in pairs(bar.spells) do
				if type(spells) == "table" then
					for j, inSpell in pairs(spells) do
						if inSpell == spell then return end
					end
				else
					if spell == spells then return end
				end
			end
		end
	end
	
	local bar = self:CreateBar()
	bar.type = "SpellStackCombat"
	bar.spells = type(spells) ~= "table" and {spells} or spells
	local spellName, rank, icon = GetSpellInfo(bar.spells[1])
	bar.spellName = spellName
	bar.spellIcon = icon
	bar.maxStacks = maxStacks or 20
	bar.typeData = id
	bar.icon:SetTexture(icon)
	
	bar.strName:SetText(spellName)
	bar.strBottom:SetText("0 / "..bar.maxStacks)
	bar.bar:SetValue(0)
	
	if markers and type(markers) == "table" then
		plugin:HandleMarkers(anchor.db, bar, markers)
	end
	
	anchor:AddBar(bar)
	
	bar:Show()
end

function plugin:MPBSpellStackHide(event, id)
	for anchorName, anchor in pairs(Anchors) do
		for i, bar in pairs(anchor.bars) do
			if bar.typeData == id and (bar.type == "SpellStackCombat" or bar.type == "SpellStackAura") then
				bar:Hide()
			end
		end
	end
end

function plugin:MPBExternal(event, id, func, anchorID, markers, icon)
	if not func or type(func) ~= "function" then return end
	local min, max, name = func()
	if (not min or type(min) ~= "number") or (not max or type(max) ~= "number") or (not name or type(name) ~= "string") then error(("Invalid external function - return valaues must be %q, %q, %q"):format("min", "max", "name")) end
	local anchor = plugin:GetAnchor(anchorID)
	
	for i, bar in pairs(anchor.bars) do
		if bar.type == "External" and bar.id == id and not bar.stopping then return end
	end
	
	local bar = self:CreateBar()
	bar.type = "External"
	bar.id = id
	bar.typeData = func
	
	if not icon then
		bar.NoIcon = true
	else
		bar.icon:SetTexture(icon)
	end
	
	if markers and type(markers) == "table" then
		plugin:HandleMarkers(anchor.db, bar, markers)
	end
	
	anchor:AddBar(bar)
	
	bar:Show()
end

function plugin:MPBExternalHide(event, id)
	for anchorName, anchor in pairs(Anchors) do
		for i, bar in pairs(anchor.bars) do
			if bar.type == "External" and bar.id == id then
				bar:Hide()
			end
		end
	end
end

function plugin:MPBHideAll(event)
	for anchorName, anchor in pairs(Anchors) do
		for i, bar in pairs(anchor.bars) do			
			bar:Hide()
		end
	end
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~
-- Anchors ~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~

do
	local anchorProt = {}
	
	function plugin:CreateAnchor(name)
		if not Anchors[name] then
			local f = Utils:CreateAnchor(name, UIParent, "Bar: "..name)
			f.name = name
			f:SetScript("OnDragStart", dragStart)
			f:SetScript("OnDragStop", dragStop)
			
			f:SetHeight(0.0001)
			
			local newAnchor = setmetatable({
				anchor = f,
				bars = {},
			}, {__index = anchorProt})
			Anchors[name] = newAnchor
			
			local db = db.Anchors[name]		
			db.Name = db.Name or name
			
			newAnchor.db = db
			newAnchor:SetName(newAnchor.db.Name)
			
			f:SetWidth(db.Width)
		end	
		
		if InConfigMode then
			self:ConfigMode(nil, false)
			self:ConfigMode(nil, true)
		end
		
		Anchors[name]:ApplySettings()
		
		return Anchors[name]
	end
	
	function anchorProt:SetName(name)
		self.anchor.name = name
		self.db.Name = name
		self:SetText("Multi-Purpose Bar: "..name)
	end
	
	function anchorProt:SetText(value)
		self.anchor.str:SetText(value)
	end
	
	function anchorProt:Show()
		Utils.FadeIn(self.anchor)
	end
	
	function anchorProt:Hide()
		Utils.FadeOut(self.anchor)
	end
	
	function anchorProt:Delete()
		for i, bar in pairs(self.bars) do		
			bar:Hide()
		end	
		self:Hide()
	end
	
	function anchorProt:AddBar(bar)
		tinsert(self.bars, bar)
		bar.anchor = self
		bar.db = self.db
		
		self:SortBars()
	end
	
	function anchorProt:RemoveBar(barToRemove)
		for i, bar in pairs(self.bars) do
			if bar == barToRemove then
				table.remove(self.bars, i)
				self:SortBars()
			end
		end		
	end
	
	function anchorProt:SortBars()
		for i, bar in pairs(self.bars) do
			bar.order = i - 1
			bar:ApplySettings()
		end
	end
	
	function anchorProt:ApplySettings()
		local anchor = FA:GetAnchor(self.db.AnchorFrame or "BlizzUIParent")
		if not anchor then anchor = UIParent end
		
		self.anchor:ClearAllPoints()			
		self.anchor:SetPoint(self.db.Point, anchor, self.db.RelPoint, self.db.Position[1], self.db.Position[2])
		self.anchor:SetWidth(self.db.Width)
	
		for i, bar in pairs(self.bars) do
			bar:ApplySettings()
		end
	end
	
	local AnchorList = {}
	function  plugin:GetAnchorList()
		wipe(AnchorList)
		for anchorName, data in pairs(Anchors) do
			if anchorName ~= "**" then
				AnchorList[anchorName] = {text = anchorName, value = anchorName}
			end
		end
		return AnchorList
	end
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~
-- Bars ~~~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~

do
	local barProt = {}
	
	local function cID(uid)
		if not uid then return end
		local guid = UnitGUID(uid)
		if guid then return tonumber(guid:sub(7, 10), 16) end
	end
	
	local function GetUID(id)
		if Utils:IsUID(id) then return id end
		local func
		if type(id) == "number" then func = cID
		else func = UnitGUID end
		
		local uID
		if func("mouseover") == id then uID = "mouseover"
		elseif func("target") == id then uID = "target"
		elseif func("focus") == id then uID = "focus"
		elseif func("player") == id then uID = "player"
		else
			for name, data in RW.Party:PartyIterator() do
				if func(data.id.."target") == id then uID = data.id.."target" end
			end
		end
		
		return uID
	end
	
	local function BarClicked(self, arg1, arg2)
		if arg1 == "LeftButton" then
			local party = RW.Party
			local pType = ((party:GetPartyType() == "raid" and "RAID") or (party:GetPartyType() == "party" and "PARTY") or (party:GetPartyType() == "battleground" and "BATTLEGROUND")) or "SAY"
			if pType then
				SendChatMessage("[RW]: "..self.obj.strName:GetText().." - " ..self.obj.strBottom:GetText(), pType)
			end
		else
			self.obj:Hide()
		end
	end
	
	local function BarEvent(self, event, arg1, arg2)
		if arg1 == "LSHIFT" or arg1 == "RSHIFT" then
			if arg2 == 1 then
				self:EnableMouse(true)
			else
				self:EnableMouse(false)
			end
		end
	end
	
	local function BarHide(self)
		self.obj.anchor:RemoveBar(self.obj)
		tinsert(BarCache, self.obj)
		self.obj.type = "none"
		self.obj.typeData = nil
		self.lastMarkIndex = nil
		self.obj.anchor = nil
		self.obj.db = nil
		self.obj.order = nil
		self.obj.DataFound = nil
		self.obj.NoIcon = nil
		self.obj.HaveModel = nil
		self.obj.id = nil
		self.obj.IconOverride = nil
		self.obj.icon:SetTexture(nil)
		self.obj.icon:Show()
		self.obj.model:Hide()
		self.obj.strName:SetText()
		self.obj.strBottom:SetText()
		
		self.obj.spell = nil
		self.obj.spells = nil
		self.obj.spellName = nil
		self.obj.spellIcon = nil
		self.obj.maxStacks = nil
		self.obj.buffType = nil
		
		self.obj.eventFrame:UnregisterAllEvents()
		self.obj.eventFrame:SetScript("OnEvent", nil)
		
		self.obj.stopping = nil
		
		if self.obj.markers then
			for i, marker in pairs(self.obj.markers) do
				marker.percent = nil
				marker:ClearAllPoints()
				marker:Hide()
				tinsert(MarkerCache, marker)
			end
			self.obj.markers = nil
		end
	end
	
	function plugin:CreateBar()
		local newBar = table.remove(BarCache)
		
		if not newBar then
			local frame = CreateFrame("Frame", nil, UIParent)
			frame:SetScript("OnEvent", BarEvent)
			frame:RegisterEvent("MODIFIER_STATE_CHANGED")
			frame:SetScript("OnMouseDown", BarClicked)
			frame:SetScript("OnHide", BarHide)
			frame:SetScript("OnUpdate", barProt.Update)
			
			local textLevel = CreateFrame("Frame", nil, frame)
			textLevel:SetFrameLevel(textLevel:GetFrameLevel() + 10)
			
			local eventFrame = CreateFrame("Frame", nil, frame)
			
			local bg = frame:CreateTexture(nil, "BACKGROUND")
			
			local bar = CreateFrame("StatusBar", nil, frame)
			bar:SetFrameLevel(bar:GetFrameLevel() + 1)
			bar:SetMinMaxValues(0, 1)
			
			local icon = frame:CreateTexture(nil, "ARTWORK")
			icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
			
			local raidMark = frame:CreateTexture(nil, "OVERLAY")
			raidMark:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons.blp")
			raidMark:Hide()
			
			local model = CreateFrame("PlayerModel", nil, frame)
			model:Hide()
			
			local strName = textLevel:CreateFontString(nil, nil, "GameFontNormal")
			strName:SetJustifyH("LEFT")
			
			local strBottom = textLevel:CreateFontString(nil, nil, "GameFontNormal")
			strBottom:SetJustifyH("LEFT")
			
			newBar = setmetatable({
				frame = frame,
				bg = bg,
				bar = bar,
				icon = icon,
				raidMark = raidMark,
				model = model,
				eventFrame = eventFrame,
				strName = strName,
				strBottom = strBottom,
				type = "none",
			}, {__index = barProt})
						
			frame.obj = newBar
			eventFrame.obj = newBar
			
			tinsert(AllBars, newBar)
		end
		
		newBar.frame:SetAlpha(0)
		newBar.frame:Show()
		
		return newBar
	end
	
	function barProt:Hide()
		self.stopping = true
		Utils.FadeOut(self.frame)
	end
	
	function barProt:Show()
		Utils.FadeIn(self.frame, true)
	end
	
	function barProt:UpdateIcon(uID)
		if self.db.UseModel and not self.IconOverride then
			self.icon:Hide()
			self.model:Show()
			self.model:SetUnit(uID)						
			self.model:SetCamera(self.db.Camera)	
			self.HaveModel = true
		elseif not self.IconOverride then	
			self.model:Hide()
			SetPortraitTexture(self.icon, uID)
		end
		self.DataFound = true		
		
		if not self.db.ShowIcon then
			self.model:Hide()
		end
		
		self:Show()
	end
	
	local function FormatValues(fmt, c, t, p)
		local r = fmt
		r = r:gsub("%%C", Utils:CondensedNumber(c or 0))
		r = r:gsub("%%c", c or 0)
		r = r:gsub("%%T", Utils:CondensedNumber(t or 0))
		r = r:gsub("%%t", t or 0)
		r = r:gsub("%%P", ("%.1f"):format(p))
		
		return r
	end
	
	function barProt:UpdateHealth(uID)
		local health, maxHealth = UnitHealth(uID), UnitHealthMax(uID)
		local p = health/maxHealth
		self.bar:SetValue(p)
		self.strName:SetText(UnitName(uID))
		self.strBottom:SetText(FormatValues(self.db.FormatString, health, maxHealth, p * 100))
	end
	
	function barProt:UpdatePower(uID)
		if self.alternate then
			local power, powerMax = UnitPower(uID, ALTERNATE_POWER_INDEX), UnitPowerMax(uID, ALTERNATE_POWER_INDEX)
			local p = power / powerMax
			local barType, minPower, startInset, endInset, smooth, hideFromOthers, showOnRaid, opaqueSpark, opaqueFlash, powerName, powerTooltip = UnitAlternatePowerInfo(uID)
			self.bar:SetValue(p)
			self.strName:SetText(powerName)
			self.strBottom:SetText(FormatValues(self.db.FormatString, power, powerMax, p * 100))
		else
			local power, powerMax = UnitPower(uID), UnitPowerMax(uID)
			local p = power / powerMax
			self.bar:SetValue(p)
			self.strName:SetText(UnitName(uID))
			self.strBottom:SetText(FormatValues(self.db.FormatString, power, powerMax, p * 100))
		end
	end
	
	function barProt:UpdateSpellStack(uID)
		local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitAura(uID, self.spellName, nil, self.buffType)
		if name then
			local p = count / self.maxStacks
			self.bar:SetValue(p)				
			self.strBottom:SetText(FormatValues(self.db.FormatString, count, self.maxStacks, p * 100))
		else
			self.bar:SetValue(0)
			self.strBottom:SetText(FormatValues(self.db.FormatString, 0, self.maxStacks, 0))
		end
	end
	
	local function SpellStackCombatEvent(self, _, _, event, hCaster, sGUID, sName, sFlags, dGUID, dName, dFlags, ...)
		local self = self.obj
		if event:sub(1, 10) ~= "SPELL_AURA" or event == "SPELL_AURA_REFRESH" then return end
		local targetID
		if type(self.typeData) == "number" then targetID = cID(dGUID) else targetID = dGUID end
		if targetID ~= self.typeData then return end
		local spellID, spellName, spellSchool, buffType, amount = ...
		
		local correctSpell = false
		for i, trackSpell in pairs(self.spells) do
			if trackSpell == spellID then correctSpell = true end
		end
		if not correctSpell then return end
		
		if event == "SPELL_AURA_REMOVED" then
			self.strBottom:SetText(FormatValues(self.db.FormatString, 0, self.maxStacks, 0))
			self.bar:SetValue(0)
		elseif event == "SPELL_AURA_APPLIED" then
			local p = 1 / self.maxStacks
			self.strBottom:SetText(FormatValues(self.db.FormatString, 1, self.maxStacks, p * 100))	
			self.bar:SetValue(p)
		else
			local p = amount / self.maxStacks
			self.strBottom:SetText(FormatValues(self.db.FormatString, amount, self.maxStacks, p * 100))
			self.bar:SetValue(p)
		end
	end
	
	function barProt:UpdateExternal()
		local cur, max, name = self.typeData()
		local p = cur / max
		self.bar:SetValue(p)			
		self.strBottom:SetText(cur.." / "..max)
		self.strBottom:SetText(FormatValues(self.db.FormatString, cur, max, p * 100))
		self.strName:SetText(name)
	end
	
	function barProt.Update(self, elapsed)
		self = self.obj
		if self.type == "none" then return end
		
		if Utils:IsUID(self.typeData) then
			-- Ticket #72 - Health frames sticking around. Hide when the uID is missing.
			if not UnitExists(self.typeData) then
				self:Hide()
				return
			end
			local targetIndex = GetRaidTargetIndex(self.typeData)
			if targetIndex then					
				if targetIndex ~= self.lastMarkIndex then
					self.lastMarkIndex = targetIndex
					self.raidMark:Show()
					self.raidMark:SetTexCoord(unpack(iconCoords[targetIndex]))
				end
			else
				self.raidMark:Hide()
				self.lastMarkIndex = nil
			end
		end
		
		if self.type == "HealthID" then
			local uID = GetUID(self.typeData)
			if uID then
				if not self.DataFound then				
					self:UpdateIcon(uID)
				end
				self:UpdateHealth(uID)
			end
			
		elseif self.type == "PowerID" then 
			local uID = GetUID(self.typeData)
			if uID then
				if not self.DataFound then				
					self:UpdateIcon(uID)
				end
				self:UpdatePower(uID)
			end
			
		elseif self.type == "SpellStackAura" then
			local uID = GetUID(self.typeData)
			if uID then
				self:UpdateSpellStack(uID)
			end
		
		elseif self.type == "SpellStackCombat" then
			if not self.eventFrame:IsEventRegistered("COMBAT_LOG_EVENT_UNFILTERED") then
				self.eventFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
				self.eventFrame:SetScript("OnEvent", SpellStackCombatEvent)
			end
			
		elseif self.type == "External" then
			self:UpdateExternal()
		end
	end
	
	function barProt:ApplySettings()
		-- Clear Points
		self.frame:ClearAllPoints()
		self.raidMark:ClearAllPoints()
		self.bg:ClearAllPoints()
		self.bar:ClearAllPoints()
		self.icon:ClearAllPoints()
		self.model:ClearAllPoints()
		self.strName:ClearAllPoints()
		self.strBottom:ClearAllPoints()
		
		-- Set Points
		if self.db.GrowUp then
			self.frame:SetPoint("BOTTOMLEFT", self.anchor.anchor, "TOPLEFT", 0, self.db.Height * self.order + self.db.VerticalSpacing * self.order)		
		else
			self.frame:SetPoint("TOP", self.anchor.anchor, "BOTTOM", 0, -(self.db.Height * self.order - self.db.VerticalSpacing * self.order))		
		end
		self.bg:SetPoint("TOPLEFT", self.db.Inset, -self.db.Inset)
		self.bg:SetPoint("BOTTOMRIGHT", -self.db.Inset, self.db.Inset)
		if self.db.IconsSide == "LEFT" then
			if self.db.ShowIcon and not self.NoIcon then
				self.icon:SetPoint("TOPLEFT", self.bg)	
				self.model:SetPoint("TOPLEFT", self.bg)	
				self.bar:SetPoint("TOPLEFT", self.icon, "TOPRIGHT", 1, 0)
			else
				self.bar:SetPoint("TOPLEFT", self.bg, 0, 0)
			end
			self.bar:SetPoint("BOTTOMRIGHT", self.bg)
		else
			if self.db.ShowIcon and not self.NoIcon then
				self.icon:SetPoint("TOPRIGHT", self.bg)					
				self.model:SetPoint("TOPRIGHT", self.bg)				
				self.bar:SetPoint("BOTTOMRIGHT", self.icon, "BOTTOMLEFT", -1, 0)
			else
				self.bar:SetPoint("BOTTOMRIGHT", self.bg)
			end
			
			self.bar:SetPoint("TOPLEFT", self.bg)
		end
		self.icon:SetPoint("BOTTOM", self.bg)	
		self.model:SetPoint("BOTTOM", self.bg)				
		if self.db.PercentSide == "RIGHT" then
			self.strBottom:SetPoint("RIGHT", self.bar, -4, 0)
			self.strName:SetPoint("LEFT", self.bar, 5, 0)
			--self.strName:SetPoint("BOTTOMLEFT", self.bar, 5, self.db.Height / 2)
			--self.strName:SetPoint("TOPLEFT", self.bar, 5, 0)
			--self.strBottom:SetPoint("TOPLEFT", self.bar, 5, -self.db.Height / 2)
			--self.strBottom:SetPoint("BOTTOMLEFT", self.bar, 5, 0)
			self.strName:SetJustifyH("LEFT")
			self.strBottom:SetJustifyH("RIGHT")			
		else
			self.strBottom:SetPoint("LEFT", self.bar, 4, 0)
			self.strName:SetPoint("RIGHT", self.bar, -5, 0)
			--self.strName:SetPoint("BOTTOMRIGHT", self.bar, -5, self.db.Height / 2)
			--self.strName:SetPoint("TOPRIGHT", self.bar, -5, 0)
			--self.strBottom:SetPoint("TOPRIGHT", self.bar, -5, -self.db.Height / 2)		
			--self.strBottom:SetPoint("BOTTOMRIGHT", self.bar, -5, 0)
			
			self.strName:SetJustifyH("RIGHT")
			self.strBottom:SetJustifyH("RIGHT")
		end

		-- Set Font
		self.strName:SetFont(LSM:Fetch("font", self.db.FontSettings.Font), self.db.FontSettings.Size, self.db.FontSettings.Flags)
		self.strBottom:SetFont(LSM:Fetch("font", self.db.FontSettings.Font), self.db.FontSettings.Size, self.db.FontSettings.Flags)
		
		-- Set Textures
		self.strName:SetTextColor(unpack(self.db.FontSettings.Color))
		self.strBottom:SetTextColor(unpack(self.db.FontSettings.Color))	
		self.bg:SetTexture(unpack(self.db.BackgroundColor))
		self.bar:SetStatusBarTexture(LSM:Fetch("statusbar", self.db.StatusbarTexture))
		self.bar:GetStatusBarTexture():SetHorizTile(false)		
		
		backdrop.edgeFile = LSM:Fetch("border", self.db.BorderTexture)
		backdrop.edgeSize = self.db.BorderWidth
		self.frame:SetBackdrop(backdrop)
		
		-- Set Colors
		self.bar:SetStatusBarColor(unpack(self.db.StatusbarColor))		
		self.frame:SetBackdropBorderColor(unpack(self.db.BorderColor))
		
		
		-- Set Size
		self.frame:SetHeight(self.db.Height)
		self.frame:SetWidth(self.db.Width)
		if self.db.ShowIcon and not self.NoIcon then
			self.model:SetWidth(self.db.Height - self.db.Inset * 2) 
			self.icon:SetWidth(self.db.Height - self.db.Inset * 2) 			
			if self.HaveModel and self.db.UseModel then
				self.model:Show()
			else
				self.icon:Show()
			end
		else
			self.model:SetWidth(0) 
			self.icon:SetWidth(0) 
			self.model:Hide()
			self.icon:Hide()
		end
		
		-- Model Camera
		self.model:SetCamera(self.db.Camera)
		
		self.raidMark:SetPoint("RIGHT", self.frame, "LEFT")
		self.raidMark:SetSize(self.db.Height, self.db.Height)
		
		-- Markers
		if self.markers then
			for i, marker in pairs(self.markers) do
				marker:Show()
				marker:SetWidth(self.db.MarkerWidth)				
				marker:SetHeight(self.db.Height - self.db.Inset * 2)
				marker.texture:SetTexture(unpack(self.db.MarkerColor))
				marker:SetPoint("LEFT", self.bar, self.bar:GetWidth() / 100 * marker.percent - self.db.MarkerWidth / 2, 0)
				marker:SetParent(self.bar)
			end
		end
	end
end

do
	function plugin:CreateMarker()
		local marker = table.remove(MarkerCache)
		if not marker then
			marker = CreateFrame("Frame", nil, UIParent)
			
			local t = marker:CreateTexture(nil, "OVERLAY")
			t:SetAllPoints()

			marker.texture = t
		end
		
		return marker
	end
	
	function plugin:HandleMarkers(db, bar, markers)
		if not db.ShowMarkers then return end
		for i, marker in pairs(markers) do
			local m = self:CreateMarker()
			m.percent = marker
			bar.markers = bar.markers or {}
			tinsert(bar.markers, m)
		end
	end
end


-- ~~~~~~~~~~~~~~~~~~~~~~~~
-- Options ~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~
function plugin:ConfigMode(event, start)
	if start then
		for i, anchor in pairs(Anchors) do
			RW.Callbacks:Fire("MPBHealthByID", UnitGUID("player"), i, {25, 50, 75})
			RW.Callbacks:Fire("MPBExternal", "id1", function() return 4, 13, "Test" end, i, {50}, select(3, GetSpellInfo(51714)))
			RW.Callbacks:Fire("MPBExternal", "id2", function() return 9, 13, "Test2" end, i, {25, 75}, select(3, GetSpellInfo(16827)))
			anchor:Show()
			anchor.anchor:SetHeight(25)
		end
	else
		for i, anchor in pairs(Anchors) do
			RW.Callbacks:Fire("MPBHealthHideByID", UnitGUID("player"))
			RW.Callbacks:Fire("MPBExternalHide", "id1")
			RW.Callbacks:Fire("MPBExternalHide", "id2")
			anchor:Hide()
			anchor.anchor:SetHeight(0.0001)
		end
	end
end

function plugin:GetOptions()	
	return {{text = LB.PLUGINS, value = "Plugins"}, {text = LB.ALERT_PLUGINS, value = "AlertPlugins"}, {text = L.NAME, value = "MPG"}}, self.GetGUI, [[Interface\Icons\INV_Sigil_Mimiron]]
end

do
	local function ChangeCallback()
		plugin:Reset()
	end
	
	local AllAnchorsList = {}
	local CopyAnchorSettingsList = {}
	-- List tables for dropdowns
	local SideList = {
		["LEFT"] = "Left",
		["RIGHT"] = "Right",
	}
	local CameraList = {
		[0] = "Face",
		[1] = "Full Body",
	}
	-- Function
	local AnchorSelected
	local TabSelected
	-- Others
	local LastSelectedAnchor
	local LastSelectedTab
	local TheAnchor
	
	local function AnchorCallback()
		TheAnchor:ApplySettings(true)
	end

	function plugin:GetGUI()
		UIF = LibStub("LibGUIFactory-1.0"):GetFactory("RW")	
		local self = self or plugin		
		
		local scroll = AceGUI:Create("SimpleGroup")	
		scroll:SetFullWidth(true)
		scroll:SetLayout("Flow")
		
		local label = UIF:MainTitleLabel(L.NAME)
		scroll:AddChild(label)
		
		local group = UIF:InlineGroup("")
		scroll:AddChild(group)	
		
		label = UIF:NormalText(L.DESC)
		label:SetImage([[Interface\Icons\INV_Sigil_Mimiron]])
		group:AddChild(label)
		
		local ed = AceGUI:Create("EditBox")
		ed:SetLabel(L.CREATE_ANCHOR)
		ed:SetCallback("OnEnterPressed", function(self, event, text)
			local newAnchor = plugin:CreateAnchor(text)
			LastSelectedAnchor = text
			newAnchor:Show()
			self:SetText()
			self.editbox:ClearFocus()
			RW.Callbacks:Fire("RefreshOptions")
		end)
		scroll:AddChild(ed)
		
		scroll:AddChild(UIF:Button(L.SHOW_ANCHORS, function()
			for i, anchor in pairs(Anchors) do
				anchor:Show()
			end
		end))		
		
		scroll:AddChild(UIF:Button(L.HIDE_ANCHORS, function()
			for i, d in pairs(Anchors) do
				d:Hide()
			end
		end))		
		
		scroll:AddChild(UIF:Button(LB.RESET, function() 
			plugin.db:ResetProfile() 
			RW.Callbacks:Fire("RefreshOptions")
		end))
		
		group:AddChild(UIF:CheckBox(LB.ENABLED, db, "PluginEnabled", function() 
			if not db.PluginEnabled then
				self:Disable()
			else
				self:Enable()
			end
		end, nil, 0.3))
		
		-- Main anchor select
		wipe(AllAnchorsList)
		for anchorName, data in pairs(Anchors) do
			if anchorName ~= "**" then
				AllAnchorsList[anchorName] = {text = data.db.Name, value = anchorName}
			end
		end
		
		local drop = UIF:DropdownGroup("Anchors")
		scroll:AddChild(drop)
		drop:SetLayout("Flow")
		drop:SetGroupList(AllAnchorsList)
		drop:SetCallback("OnGroupSelected", AnchorSelected)
		drop:SetGroup((LastSelectedAnchor and Anchors[LastSelectedAnchor]) and LastSelectedAnchor or "Default")	
		
		return scroll
	end
	
	AnchorSelected = function(self, event, group)
		self:ReleaseChildren()
		LastSelectedAnchor = group
		
		TheAnchor = Anchors[group]
		
		local tgroup = UIF:TabGroup1()
		self:AddChild(tgroup)
		tgroup:SetTabs(
		{
			{text = LB.POSITION .." & " .. LB.SIZE, value = "Position"},
			{text = LB.APPEARANCE, value = "Appearance"},
			{text = LB.LAYOUT, value = "Layout"},
			{text = "Other", value = "Handle"},
		})
		tgroup:SetCallback("OnGroupSelected", TabSelected)
		tgroup:SelectTab(LastSelectedTab or "Position")
		
		Utils:DoLayout(tgroup)
	end
	
	TabSelected = function(self, event, tab)
		self:ReleaseChildren()
		LastSelectedTab = tab
		
		if tab == "Position" then
			local group = UIF:InlineGroup1(LB.POSITION)
			self:AddChild(group)
			
			local t = AceGUI:Create("FrameAnchorDropdown")
			t:SetRelativeWidth(0.5)
			t:SetList(FA:GetAnchors(TheAnchor.anchor))
			t:SetLabel("Anchor")
			t:SetValue(TheAnchor.db.AnchorFrame)
			t:SetCallback("OnValueChanged", function(self, event, cat, name, frame)
				TheAnchor.db.AnchorFrame = cat
				AnchorCallback()
			end)	
			group:AddChild(t)
			group:AddChild(UIF:NewLine())
			
			group:AddChild(UIF:GridSelect("Point", TheAnchor.db, "Point", "Anchor", 3, AnchorCallback, 0.5, 75))
			group:AddChild(UIF:GridSelect("Relative Point", TheAnchor.db, "RelPoint", "Anchor", 3, AnchorCallback, 0.5, 75))
			
			local slider = UIF:Slider("X", TheAnchor.db.Position, 1, 
				-floor(GetScreenWidth() * UIParent:GetEffectiveScale()), 
				floor(GetScreenWidth() * UIParent:GetEffectiveScale()),
				1, AnchorCallback, 0.5)
			group:AddChild(slider)
			
			slider = UIF:Slider("Y", TheAnchor.db.Position, 2, 
				-floor(GetScreenHeight() * UIParent:GetEffectiveScale()), 
				floor(GetScreenHeight() * UIParent:GetEffectiveScale()),
				1, AnchorCallback, 0.5)
			group:AddChild(slider)
			
			group = UIF:InlineGroup(LB.SIZE)
			self:AddChild(group)
			
			group:AddChild(UIF:Slider(LB.WIDTH, TheAnchor.db, "Width", 5, 800, 1, AnchorCallback, 0.5))
			group:AddChild(UIF:Slider(LB.HEIGHT, TheAnchor.db, "Height", 5, 800, 1, AnchorCallback, 0.5))
		
		elseif tab == "Appearance" then
			self:AddChild(UIF:CheckBox(L.USE_MODEL, TheAnchor.db, "UseModel", AnchorCallback, "Will only take effect for new frames", 0.5))
			self:AddChild(UIF:Dropdown("Camera", TheAnchor.db, "Camera", AnchorCallback, CameraList, 0.5))
			
			-- Bars
			local group = UIF:InlineGroup2(LB.BARS)
			self:AddChild(group)
			
			group:AddChild(UIF:ColorSelect(LB.BACKGROUND_COLOR, TheAnchor.db, "BackgroundColor", AnchorCallback, true, 0.5))			
			group:AddChild(UIF:ColorSelect(LB.BAR_COLOR, TheAnchor.db, "StatusbarColor", AnchorCallback, true, 0.5))		
			group:AddChild(UIF:StatusbarSelect(LB.BAR_TEXTURE, TheAnchor.db, "StatusbarTexture", AnchorCallback, 0.5))
			group:AddChild(UIF:Slider(LB.ALPHA, TheAnchor.db, "Alpha", 0, 1, 0.01, AnchorCallback, 0.5))
			
			-- Border
			group = UIF:InlineGroup2(LB.BORDER)
			self:AddChild(group)
			
			group:AddChild(UIF:ColorSelect(LB.BORDER_COLOR, TheAnchor.db, "BorderColor", AnchorCallback, true, 0.33))		
			group:AddChild(UIF:Slider(LB.WIDTH, TheAnchor.db, "BorderWidth", 0, 50, 1, AnchorCallback, 0.33))
			group:AddChild(UIF:LSMDropdown("border", LB.BORDER_TEXTURE, TheAnchor.db, "BorderTexture", AnchorCallback, 0.33))						
			
			-- Font
			self:AddChild(UIF:FontSettings(LB.FONT_MAIN, TheAnchor.db.FontSettings, true, AnchorCallback, 1))
			
			-- Markers
			local group = UIF:InlineGroup2("Markers")
			self:AddChild(group)
			
			group:AddChild(UIF:CheckBox("Show Markers", TheAnchor.db, "ShowMarkers", AnchorCallback, "Will show markers on on the bars if they are provided. Example would be phase changes on bosses by certain percent HP. Note that it will on affect new created bars"))
			group:AddChild(UIF:ColorSelect(LB.COLOR, TheAnchor.db, "MarkerColor", AnchorCallback, true, 0.3))
			group:AddChild(UIF:Spacer(10))
			group:AddChild(UIF:Slider(LB.WIDTH, TheAnchor.db, "MarkerWidth", 1, 50, 0.1, AnchorCallback, 0.6))
	
		elseif tab == "Layout" then
			self:AddChild(UIF:CheckBox(LB.GROW_UP, TheAnchor.db, "GrowUp", AnchorCallback, LB.GROW_UP_DESC, 0.5))			
			self:AddChild(UIF:CheckBox(L.SHOW_ICON, TheAnchor.db, "ShowIcon", AnchorCallback, L.SHOW_ICON_DESC, 0.5))
			self:AddChild(UIF:Slider(LB.VERTICAL_SPACING, TheAnchor.db, "VerticalSpacing", -20, 40, 1, AnchorCallback, 0.5))			
			self:AddChild(UIF:Slider(LB.INTERNAL_SPACING, TheAnchor.db, "Inset", -20, 40, 1, AnchorCallback, 0.5))							
			self:AddChild(UIF:Dropdown(L.ICON_POSITION, TheAnchor.db, "IconsSide", AnchorCallback, SideList, 0.5))
			self:AddChild(UIF:Dropdown("Side for values", TheAnchor.db, "PercentSide", AnchorCallback, SideList, 0.5))
						
			local e = AceGUI:Create("EditBox")
			e:SetRelativeWidth(0.6)
			e:SetLabel("Format String")
			e:SetText(TheAnchor.db.FormatString)
			e:SetCallback("OnEnterPressed", function(self, event, value)
				TheAnchor.db.FormatString = value
				AceGUI:ClearFocus()
				AnchorCallback()
			end)
			self:AddChild(e)
			
			self:AddChild(UIF:Text1(
[[Set the format of the information displayed on bars. 
Values that get replaced: 
 |cffff0000%C|r: Current value condensed (1.2m, 32.4k)
 |cffff0000%c|r: Current value (1200453, 32400)
 |cffff0000%T|r: Max value condensed (3.2m, 68.5k)
 |cffff0000%t|r: Max value (3200512, 68500)
 |cffff0000%P|r: Percent value (50.5, 100.0) ]], 
			nil, nil, nil, nil, nil, 0.8))
			
		elseif tab == "Handle" then
			self:SetLayout("List")
			local copyDisabled = true
			wipe(CopyAnchorSettingsList)
			for anchorName, data in pairs(Anchors) do
				if anchorName ~= "**" and anchorName ~= LastSelectedAnchor then
					CopyAnchorSettingsList[anchorName] = {text = data.db.Name, value = anchorName}
					copyDisabled = false
				end
			end
			
			local button = UIF:Button(LB.DELETE, function(self)
				db.Anchors[LastSelectedAnchor] = nil
				Anchors[LastSelectedAnchor] = nil
				TheAnchor:Delete()
			end)
			button:SetUserData("confirm", true)
			button:SetUserData("TriggerGlobalUpdate", true)
			button:SetDisabled(LastSelectedAnchor == "Default" or LastSelectedAnchor == "Health Frames")
			self:AddChild(button)
			
			local ed = AceGUI:Create("EditBox")
			ed:SetLabel(LB.RENAME)
			ed:SetDisabled(LastSelectedAnchor == "Default" or LastSelectedAnchor == "Health Frames")
			ed:SetCallback("OnEnterPressed", function(self, event, text)
				TheAnchor:SetName(text)
				self:SetText()
				self.editbox:ClearFocus()
				RW.Callbacks:Fire("RefreshOptions")
			end)
			self:AddChild(ed)
			
			local drop = UIF:SimpleDropdown(L.COPY_SETTINGS, function(self, event, value)
				local oldX = TheAnchor.db.Position[1]
				local oldY = TheAnchor.db.Position[2]
				local oldName = TheAnchor.db.Name
				Utils:CopyTableShallow(Anchors[value].db, TheAnchor.db)
				TheAnchor.db.Position[1] = oldX
				TheAnchor.db.Position[2] = oldY
				TheAnchor.db.Name = oldName
				TheAnchor:ApplySettings()
				RW.Callbacks:Fire("RefreshOptions")
			end, CopyAnchorSettingsList)
			drop:SetDisabled(copyDisabled)
			drop:SetUserData("confirm", true)
			drop:SetUserData("TriggerGlobalUpdate", true)
			self:AddChild(drop)
			
			self:AddChild(UIF:CheckBox(L.HIDE_BLIZZ, db, "HideBlizzFrames", function() plugin:Reset() end))
		end
		
		Utils:DoLayout(self)
	end
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~
-- Positioning ~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~

dragStart = function(self)
	self:StartMoving()
end

dragStop = function(self)
	self:StopMovingOrSizing()
	local scale = self:GetEffectiveScale() / UIParent:GetEffectiveScale()
	
	local x, y = self:GetCenter()
	x, y = x * scale, y * scale
	
	x = x - GetScreenWidth()/2
	y = y - GetScreenHeight()/2
	x = x * scale
	y = y * scale
	
	local aDb = db.Anchors[self.name]
	aDb.AnchorFrame = nil
	aDb.Point = "CENTER"
	aDb.RelPoint = "CENTER"
	
	aDb.Position[1], aDb.Position[2] = x, y
	
	RW.Callbacks:Fire("RefreshOptions")
end